#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define _XOPEN_SOURCE
#include <time.h>
#include <unistd.h>
#include <mxml.h>

#define	FIVE_MINUTES	(300)

static int toGPX(unsigned int, const char *);

/*
** generate minimal gpx file from range-limited co-ordinates
*/

int main(int argc,char *argv[])
{
  int status = 1;

  char *name = NULL;
  unsigned int interval = FIVE_MINUTES;

  {
    extern char *optarg;
    extern int optind, opterr, optopt;
    int opt;

    while ((opt = getopt(argc,argv,"n:i:")) != -1)
    {
      switch (opt)
      {
	case 'n':
	  free(name);
	  name = strdup(optarg);
	  break;

	case 'i':
	  {
	    unsigned int t1;

	    if (sscanf(optarg,"%u",&t1) != 1)
	    {
	      fprintf(stderr,"%s: Bad interval %s\n",argv[0],optarg);
	      status = 0;
	    }
	    else if (t1 == 0)
	    {
	      fprintf(stderr,"%s: Interval cannot be zero\n",argv[0]);
	      status = 0;
	    }
	    else interval = t1;
	  }
	  break;

	default:
	  status = 0;
	  break;
      }
    }

    if (optind != argc)
    {
      fprintf(stderr,"%s: Unexpected arguments\n",argv[0]);
      status = 0;
    }
  }

  if (status)
  {
    if (name == NULL) name = strdup("Track");
    status = toGPX(interval, name);
    free(name);
  }
  else fprintf(stderr,"Usage: %s [-i interval] [-n name]\n",argv[0]);

  return status ? EXIT_SUCCESS : EXIT_FAILURE;
}


static int toGPX(const unsigned int interval, const char *name)
{
  int		status,
		nscanned;
  unsigned int	nlocs = 0;

  mxml_node_t	*tree,
		*root;
  double	lat,			/* latitude  */
		lon,			/* longitude */
		alt;			/* altitude  */
  long int	timestamp;		/* timestamp as integer */
  time_t 	datetime_prev,		/* previous reading date/time value */
		datetime_curr = 0;	/* current reading date/time value */

  tree = mxmlNewXML("1.0");
  root = mxmlNewElement(tree,"gpx");
  mxmlElementSetAttr(root,
		     "version",
		     "1.0");
  mxmlElementSetAttr(root,
		     "creator",
		     "trkgpx");
  mxmlElementSetAttr(root,
		     "xmlns:xsi",
		     "http://www.w3.org/2001/XMLSchema-instance");
  mxmlElementSetAttr(root,
		     "xsi:schemaLocation",
		     "http://www.topografix.com/GPX/1/0 "
		     "http://www.topografix.com/GPX/1/0/gpx.xsd");


  while ((nscanned = scanf("%ld,%lf,%lf,%lf",&timestamp,&lat,&lon,&alt)) == 4)
  {
    mxml_node_t	*trk,		/* node under which all the <trkseg>s live */
		*trkseg,	/* node under which all the <trkpt>s live */
      		*trkpt;		/* single trackpoint */
    struct tm *Time;		/* trackpoint timestamp */

    datetime_prev = datetime_curr;
    datetime_curr = timestamp;
    Time = localtime(&datetime_curr);

    if (datetime_curr - datetime_prev > interval)
    {
      char dt[21];

      strftime(dt,sizeof dt,"%F@%T",Time);

      trk = mxmlNewElement(root,"trk");
      mxmlNewTextf(mxmlNewElement(trk,"name"),0,"%s: %s",name,dt);

      trkseg = mxmlNewElement(trk,"trkseg");
    }

    trkpt = mxmlNewElement(trkseg,"trkpt");
    mxmlElementSetAttrf(trkpt,"lat","%.6lf",lat);
    mxmlElementSetAttrf(trkpt,"lon","%.6lf",lon);

    mxmlNewReal(mxmlNewElement(trkpt,"ele"),alt);

    {
      char buffer[21];
      strftime(buffer,sizeof buffer,"%FT%TZ",Time);
      mxmlNewOpaque(mxmlNewElement(trkpt,"time"),buffer);
    }

    ++nlocs;
    { /* ignore rest of CSV cells and reposition at next CSV line */
      int datum;
      for (datum = getchar(); datum != EOF && datum != '\n';  datum = getchar()) continue;
      if (datum == EOF) ungetc(datum,stdin);
    }
  }

  if ((nscanned == EOF)	&& (nlocs > 0))		/* only save valid GPX */
    if (mxmlSaveFile(tree,stdout,MXML_NO_CALLBACK))
    {
      fputs("Failed to save generated GPX file\n",stderr);
      status = 0;
    }
    else
      status = 1;
  else
  {
    fputs("Failed to generate GPX file - check for valid input\n",stderr);
    status = 0;
  }

  mxmlDelete(tree);

  return status;
}

/* $Id: trkgpx.c,v 1.7 2020/06/06 16:43:04 lpitcher Exp $

    Copyright (C) 2020 Lew Pitcher <lew.pitcher@digitalfreehold.ca>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
